package rmichat;

import java.rmi.RemoteException;
import java.rmi.registry.LocateRegistry;
import java.rmi.registry.Registry;
import java.rmi.server.UnicastRemoteObject;
import java.util.Scanner;

public class RMIChatClientImpl implements RMIChatClient {

	@Override
	public void notifyMessage(String message) throws RemoteException {
		System.out.println(message);
	}

	@Override
	public void notifyDisconnection() throws RemoteException {
		System.out.println("Disconnected");
		System.exit(0);
	}
	
	public static void main(String[] args) {
    	final String REGISTRY_ADDRESS = "127.0.0.1";
    	RMIChatServer server = null;
    	RMIChatClient client = null;
    	
        try {
        	// creo il client
        	client = new RMIChatClientImpl();
        	// esporto lo stub del client (cosi' puo' essere inviato al server)
        	UnicastRemoteObject.exportObject(client, 0);

        	// prendo un riferimento al registry
            Registry registry = LocateRegistry.getRegistry(REGISTRY_ADDRESS);
            // cerco lo stub del server nel registry
            server = (RMIChatServer) registry.lookup("ChatServer");
            // registro il client nel server
            server.register(client);
            System.out.println("Client connected");
        } catch (Exception e) {
            System.err.println("Client exception: " + e.toString());
            e.printStackTrace();
            System.exit(0);
        }
        
        Scanner in = new Scanner(System.in);
        
        try {
	        while (true) {
	        	// leggo una riga
	        	String line = in.nextLine();
	        	if (line.equals("off")) {
	        		// se la riga e' "off" invio un comando di shutdown al server
	        		server.shutdown();
	        	} else {
	        		// altrimenti invio un messaggio al server
	        		server.send(client, line);
	        	}
	        }
        } catch (RemoteException e) {
        	System.out.println("Disconnected");
            System.exit(0);
        }
        

	}

}
