package rmichat;

import java.rmi.RemoteException;
import java.rmi.registry.LocateRegistry;
import java.rmi.registry.Registry;
import java.rmi.server.UnicastRemoteObject;
import java.util.List;
import java.util.concurrent.CopyOnWriteArrayList;

public class RMIChatServerImpl implements RMIChatServer {
	
	private List<RMIChatClient> clients = new CopyOnWriteArrayList<RMIChatClient>();

	@Override
	public void register(RMIChatClient client) throws RemoteException {
		// registro il client nel server (lo aggiunto alla lista dei client)
		clients.add(client);
	}

	@Override
	public void send(RMIChatClient fromClient, String message) throws RemoteException {
		// invio un messaggio a tutti i client escludendo il mittente
		for (RMIChatClient client:clients) {
			if (!client.equals(fromClient)) {
				client.notifyMessage(message);
			}
		}
	}

	@Override
	public void shutdown() throws RemoteException {
		// notifico a tutti i client la chiusura del server
		for (RMIChatClient client:clients) {
			try {
				client.notifyDisconnection();
			} catch (RemoteException e) {}
		}
		
		// termino il server
		System.exit(0);
	}
	
	public static void main(String[] args) {
		 try {
			    // creo il server
			    RMIChatServerImpl server = new RMIChatServerImpl();
			    // esporto il server e creo lo stub
			    RMIChatServer serverStub = (RMIChatServer) UnicastRemoteObject.exportObject(server, 0);
			    // prendo un riferimento al registry
	            Registry registry = LocateRegistry.getRegistry();
	            // esporto lo stub del server nel registry
	            registry.rebind("ChatServer", serverStub);

	            System.out.println("Chat Server ready");
	        } catch (Exception e) {
	            System.err.println("Chat Server exception: " + e.toString());
	            e.printStackTrace();
	        }
	}

}
