package socketchat;

import java.io.IOException;
import java.io.PrintWriter;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.List;
import java.util.Scanner;

public class ChatClientHandler implements Runnable {

	private Socket socket;
	private List<ChatClientHandler> chatClientHandlers;
	private ServerSocket serverSocket;
	private Scanner in;
	private PrintWriter out;
	
	public ChatClientHandler(Socket socket,
			List<ChatClientHandler> chatClientHandlers,
			ServerSocket serverSocket) {
		this.socket = socket;
		this.chatClientHandlers = chatClientHandlers;
		this.serverSocket = serverSocket;
	}
	
	public synchronized Socket getSocket() {
		return socket;
	}

	@Override
	public void run() {
		try {
			in = new Scanner(socket.getInputStream());
			out = new PrintWriter(socket.getOutputStream());
			
			while (true) {
				String linea = in.nextLine();
				if (linea.equals("off")) {
					closeServer();
				} else {
					sendToAll(linea);
				}
			}
			
		} catch (IOException e) {
			System.err.println(e.getMessage());
		}
	}
	
	public synchronized void send(String message) {
		if (socket.isConnected()) {
			out.println(message);
			out.flush();
		}
	}
	
	private void closeServer() throws IOException {
		synchronized (serverSocket) {
			if (!serverSocket.isClosed()) {
				serverSocket.close();
			}
		}
	}
	
	private void sendToAll(String message) {
		for (ChatClientHandler c:chatClientHandlers) {
			if (c!=this) {
				c.send(message);
			}
		}
	}

}
