package socketchat;

import java.io.IOException;
import java.net.ServerSocket;
import java.util.List;
import java.util.concurrent.CopyOnWriteArrayList;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

public class ChatServer {
	
	private List<ChatClientHandler> chatClientHandlers = new CopyOnWriteArrayList<ChatClientHandler>();
	
	public void startServer(int port) throws IOException {
		ExecutorService executor = Executors.newCachedThreadPool();
		
		ServerSocket serverSocket = new ServerSocket(port);
		System.out.println("Server avviato normalmente.");
		
		try {
			while (true) {
				ChatClientHandler clientHandler = new ChatClientHandler(serverSocket.accept(), 
						chatClientHandlers, serverSocket);
				chatClientHandlers.add(clientHandler);
				executor.submit(clientHandler);
			}
		} catch (IOException e) {
			serverSocket.close();
			executor.shutdown();
			for (ChatClientHandler c:chatClientHandlers) {
				c.getSocket().close();
			}
			chatClientHandlers.clear();
		}
	}
	
	public static void main(String[] args) {
		ChatServer chatServer = new ChatServer();
		try {
			chatServer.startServer(4747);
			System.out.println("Server terminato normalmente.");
		} catch (IOException e) {
			System.err.println(e.getMessage());
		}
	}
}
